// RelativeLayers 0.9.7
// 
// $Id: rl_layer.js 1.7 04/02/02 21:22:32+01:00 gbevin@willow.uwyn.office $
// 
// Library for creating dynamic designs that adapt themselves to the user's
// browser environment.
// 
// Copyright (C) 1998-2004, Geert Bevin
// Distributed under the terms of the GNU Lesser General Public, v2.1 or later
// 
// gbevin[remove] at uwyn dot com
// http://www.uwyn.com/projects/relativelayers

// FIXME: allow a relative layer expression (numbers + percentages) as limit offset argument
// FIXME: allow dynamic creation of nested layers, question is how to provide the syntax

// define the border identifiers
var LEFT = 0;
var RIGHT = 1;
var TOP = 2;
var BOTTOM = 3;

// array containing all relative layer objects that have been created
var glb_relativelayer_objects = new Hashtable();

function getRelativeLayer(par_id)
{
	var var_layer = glb_relativelayer_objects.get(par_id);

	return var_layer;
};

addLoadHook(initLayers, 10);
addLoadHook(initDeferredCommands, 20);
addLoadHook(initPositions, 50);

addResizeHook(resizeEnsureDivs, 10);
addResizeHook(resizeInvalidatePositioned, 30);
addResizeHook(resizeAutoReposition, 40);
addResizeHook(initPositions, 50);

function initLayers()
{
	if (mz)
	{
		initSizerLayer();
	}
	var var_layers = glb_relativelayer_objects.values();
	var var_index;

	for (var_index = 0; var_index < var_layers.length; var_index++)
	{
		var_layers[var_index].initialize();
	}
	
	return true;
};

function initSizerLayer()
{
	var var_sizerlayer = document.createElement("DIV");
	var_sizerlayer.id = 'SIZER_LAYER';
	var_sizerlayer.style.position = 'absolute';
	var_sizerlayer.style.left = '0px';
	var_sizerlayer.style.top = '0px';
	var_sizerlayer.style.width = '100%';
	var_sizerlayer.style.height = '100%';
	var_sizerlayer.style.background = 'transparent';
	var_sizerlayer.style.visibility = 'show';
	var_sizerlayer.style.zIndex = -255;
	document.body.appendChild(var_sizerlayer);
	window.SIZER_LAYER = var_sizerlayer;
};

function initDeferredCommands()
{
	var var_layers = glb_relativelayer_objects.values();
	var var_index;

	for (var_index = 0; var_index < var_layers.length; var_index++)
	{
		var_layers[var_index].processDeferredCommands();
	}

	return true;
};

function initPositions()
{
	var var_layers = glb_relativelayer_objects.values();
	var var_index;
	
	for (var_index = 0; var_index < var_layers.length; var_index++)
	{
		var_layers[var_index].position();
	}

	return true;
};

function resizeEnsureDivs()
{
	var var_layers = glb_relativelayer_objects.values();
	var var_index;

	for (var_index = 0; var_index < var_layers.length; var_index++)
	{
		if (!getCSS(var_layers[var_index].getId()))
		{
			var_layers[var_index].createDiv();
			var_layers[var_index].mbr_bgColorChanged = true;
			var_layers[var_index].mbr_bgImageChanged = true;
			var_layers[var_index].mbr_zIndexChanged = true;
		}
	}

	return true;
};

function resizeInvalidatePositioned()
{
	var var_layers = glb_relativelayer_objects.values();
	var var_index;

	for (var_index = 0; var_index < var_layers.length; var_index++)
	{
		if (var_layers[var_index].mbr_autoreposition)
		{
			var_layers[var_index].setPositioned(false);
		}
	}

	return true;
};

function resizeAutoReposition()
{
	var var_layers = glb_relativelayer_objects.values();
	var var_index;

	for (var_index = 0; var_index < var_layers.length; var_index++)
	{
		if (var_layers[var_index].mbr_autoreposition)
		{
			var_layers[var_index].position();
		}
	}

	return true;
};

function RelativeLayer(
	par_id, par_visibleHorizOffsetReference, par_visibleVertOffsetReference,
	par_visibleWidth, par_visibleHeight,
	par_visibleHorizBorder, par_visibleHorizOffset, par_visibleVertBorder, par_visibleVertOffset,
	par_visibleHandleHorizOffset, par_visibleHandleVertOffset,
	par_availableWidth, par_availableHeight,
	par_availableHorizBorder, par_availableHorizOffset, par_availableVertBorder, par_availableVertOffset,
	par_availableHandleHorizOffset, par_availableHandleVertOffset,
	par_bgColor, par_bgImage)
{
	var var_existinglayer = getRelativeLayer(par_id);

	if (var_existinglayer)
	{
		return var_existinglayer;
	}
	else
	{
		this.clearDependencies();

		if (arguments.length < 11)
		{
			alert("The constructor of the layer with id '"+par_id+"' contains less that the minimal required arguments (11).");
		}
		if (arguments.length < 12)
		{
			par_availableWidth = "100%";
		}
		if (arguments.length < 13)
		{
			par_availableHeight = "100%";
		}
		if (arguments.length < 14)
		{
			par_availableHorizBorder = "LEFT";
		}
		if (arguments.length < 15)
		{
			par_availableHorizOffset = "50%";
		}
		if (arguments.length < 16)
		{
			par_availableVertBorder = "TOP";
		}
		if (arguments.length < 17)
		{
			par_availableVertOffset = "50%";
		}
		if (arguments.length < 18)
		{
			par_availableHandleHorizOffset = "0";
		}
		if (arguments.length < 19)
		{
			par_availableHandleVertOffset = "0";
		}

		this.mbr_bgColorChanged = false;
		this.mbr_bgImageChanged = false;
		this.mbr_zIndexChanged = false;

		if (par_bgColor &&
		   par_bgColor != "")
		{
			this.setBgColor(par_bgColor);
		}
		if (par_bgImage &&
		   par_bgImage != "")
		{
			this.setBgImage(par_bgImage);
		}
		this.mbr_zIndex = false;

		this.setId(par_id);

		// initialize the variables that define wether the layer dimensions should be collection automatically
		this.mbr_autoVisibleWidth = false;
		this.mbr_autoAvailableWidth = false;
		this.mbr_autoVisibleHeight = false;
		this.mbr_autoAvailableHeight = false;

		// define the visible area
		this.defineVisibleArea(par_visibleHorizOffsetReference, par_visibleVertOffsetReference, par_visibleWidth, par_visibleHeight,
			par_visibleHorizBorder, par_visibleHorizOffset, par_visibleVertBorder, par_visibleVertOffset,
			par_visibleHandleHorizOffset, par_visibleHandleVertOffset);
		// define the available area
		this.defineAvailableArea(par_availableWidth, par_availableHeight,
			par_availableHorizBorder, par_availableHorizOffset, par_availableVertBorder, par_availableVertOffset,
			par_availableHandleHorizOffset, par_availableHandleVertOffset);

		// set the default positioning settings
		this.setPositioned(false);
		this.setAutoReposition(true);
		this.setVisible(false);

		// initialize the deferred commands array
		this.mbr_deferredcommands = new Array();

		// add this layer to the array of existing relative layers
		glb_relativelayer_objects.put(this.getId(), this);

		if (!glb_document_loaded)
		{
			this.createCSS();
		}

		return this;
	}
};

RelativeLayer.prototype.addDependency = function(par_layerid)
{
	this.mbr_dependencies.push(par_layerid);
};

RelativeLayer.prototype.removeDependency = function(par_layerid)
{
	this.mbr_dependencies.remove(par_layerid);
};

RelativeLayer.prototype.indexOfDependency = function(par_layerid)
{
	return this.mbr_dependencies.indexOf(par_layerid);
};

RelativeLayer.prototype.positionDependencies = function()
{
	var var_index;

	// first position the dependecies that are need to position this layer
	for (var_index = 0; var_index < this.mbr_dependencies.length; var_index++)
	{
		var var_dependency_layer = getRelativeLayer(this.mbr_dependencies[var_index]);
		if (var_dependency_layer)
		{
			var_dependency_layer.position();
		}
	}
};

RelativeLayer.prototype.getDependingLayers = function()
{
	var var_layers = glb_relativelayer_objects.values();
	var var_depending_layers = new Array();
	var var_index;

	// then position the layers that are dependent on this one
	for (var_index = 0; var_index < var_layers.length; var_index++)
	{
		if (var_layers[var_index].indexOfDependency(this.getId()) != -1)
		{
			var_depending_layers.push(var_layers[var_index]);
		}
	}

	return var_depending_layers;
};

RelativeLayer.prototype.positionDependingLayers = function()
{
	var var_depending_layers = this.getDependingLayers();
	var var_index;

	// then position the layers that are dependent on this one
	for (var_index = 0; var_index < var_depending_layers.length; var_index++)
	{
  		var_depending_layers[var_index].setPositioned(false);
		var_depending_layers[var_index].position();
	}
};

RelativeLayer.prototype.clearDependencies = function()
{
	this.mbr_dependencies = new Array();
};

RelativeLayer.prototype.createCSS = function()
{
	var var_csscode;

	if (op)
	{
		var_csscode =
			'<STYLE TYPE="text/css">\n'+
			'#'+this.getId()+' {position:absolute; visibility:hidden;';
			
		if (this.mbr_bgColor)
		{
			 var_csscode += 'background-color:'+this.mbr_bgColor+';';
		}
		if (this.mbr_bgImage)
		{
			 var_csscode += 'background-image:url('+this.mbr_bgImage+');';
		}
		var_csscode += ' overflow: hidden;}\n</STYLE>';
	}
	else if (ns)
	{
		var_csscode =
			'<STYLE TYPE="text/css">\n'+
			'#'+this.getId()+' {position:absolute; visibility:hidden; background-color:transparent;}\n'+
			'</STYLE>';
	}
	else if (mz || ie)
	{
		var_csscode =
			'<STYLE TYPE="text/css">\n'+
			'#'+this.getId()+' {position:absolute; visibility:hidden;}\n'+
			'</STYLE>';
	}
	else if (kq)
	{
		var_csscode =
			'<STYLE TYPE="text/css">\n'+
			'#'+this.getId()+' {position:absolute; visibility:hidden;}\n'+
			'</STYLE>';
	}
	document.write(var_csscode);
	document.close();
};

RelativeLayer.prototype.createDiv = function()
{
	if (op)
	{
		// not yet possible with opera
		unsupportedBrowser();
	}
	else if (mz || kq)
	{
		var var_div = document.createElement("DIV");
		var_div.id = this.getId();
		this.css = var_div.style;
		this.mbr_node = var_div;
		document.body.appendChild(var_div);
	}
	else if (ns)
	{
		var var_css = new Layer(this.calculateAvailableWidth());
		var_css.name = this.getId();
		var_css.visibility = "hide";
		this.css = var_css;
		this.mbr_node = this.css;
		document.layers[this.getId()] = var_css;
	}
	else if (ie)
	{
		var var_html = "\n<DIV ID=\""+this.getId()+"\">";
		document.body.insertAdjacentHTML("BeforeEnd", var_html);
		this.css = getCSS(this.getId());
		this.mbr_node = document.all[this.getId()];
	}

	// notify event listeners
	this.fireEvent("create");
};

RelativeLayer.prototype.createLayer = function()
{
	if (!this.css)
	{
		if (op)
		{
			// not yet possible with opera
			unsupportedBrowser();
		}
		else if (mz || kq)
		{
			var var_div = document.createElement("DIV");
			var_div.id = this.getId();
			var_div.style.position = "absolute";
			var_div.style.visibility = "hidden";
			this.css = var_div.style;
			this.mbr_node = var_div;
			document.body.appendChild(var_div);
		}
		else if (ns)
		{
			var var_css = new Layer(this.calculateAvailableWidth());
			var_css.name = this.getId();
			var_css.visibility = "hide";
			this.css = var_css;
			this.mbr_node = this.css;
			document.layers[this.getId()] = var_css;
		}
		else if (ie)
		{
			var var_html = "\n<DIV ID=\""+this.getId()+"\" STYLE=\"position:absolute; visibility: hidden;\">";
			document.body.insertAdjacentHTML("BeforeEnd", var_html);
			this.css = getCSS(this.getId());
			this.mbr_node = document.all[this.getId()];
		}
	
		// notify event listeners
		this.fireEvent("create");
		this.initialize();
		this.processDeferredCommands();
		this.position();
	}
};

RelativeLayer.prototype.destroyLayer = function()
{
	if (this.css)
	{
		if (op)
		{
			// not yet possible with opera
			unsupportedBrowser();
		}
		else if (mz || kq)
		{
			this.setVisible(false);
			document.body.removeChild(this.mbr_node);
			this.css = null;
			this.mbr_node = null;
			glb_relativelayer_objects.remove(this.getId());
		}
		else if (ns)
		{
			this.setVisible(false);
			this.css = null;
			this.mbr_node = null;
			glb_relativelayer_objects.remove(this.getId());
		}
		else if (ie)
		{
			this.setVisible(false);
			this.mbr_node.innerHTML = "";
			this.mbr_node.outerHTML = "";
			this.css = null;
			this.mbr_node = null;
			glb_relativelayer_objects.remove(this.getId());
		}
	}
};

RelativeLayer.prototype.setPositioned = function(par_validity)
{
	this.mbr_positioned = par_validity;
};

RelativeLayer.prototype.isPositioned = function()
{
	return this.mbr_positioned;
};

RelativeLayer.prototype.setAutoReposition = function(par_validity)
{
	this.mbr_autoreposition = par_validity;
};

RelativeLayer.prototype.initialize = function()
{
	if (op)
	{
		if (!this.css)
		{
			this.css = getCSS(this.getId());
			if (!this.css)
			{
				this.createDiv();
			}
			else
			{
				this.mbr_node = document.getElementById(this.getId());
			}
		}
		this.mbr_x = this.css.pixelLeft;
		this.mbr_y = this.css.pixelTop;
	}
	else if (mz || kq)
	{
		if (!this.css)
		{
			this.css = getCSS(this.getId());
			if (!this.css)
			{
				this.createDiv();
			}
			else
			{
				this.mbr_node = document.getElementById(this.getId());
			}
		}
		this.mbr_x = this.css.left;
		this.mbr_y = this.css.top;
	}
	else if (ns)
	{
		if (!this.css)
		{
			this.css = getCSS(this.getId());
			if (!this.css)
			{
				this.createDiv();
			}
			else
			{
				this.mbr_node = this.css;
			}
		}
		this.mbr_x = this.css.left;
		this.mbr_y = this.css.top;
	}
	else if (ie)
	{
		if (!this.css)
		{
			this.css = getCSS(this.getId());
			if (!this.css)
			{
				this.createDiv();
			}
			else
			{
				this.mbr_node = document.all[this.getId()];
			}
		}
		this.mbr_x = this.css.pixelLeft;
		this.mbr_y = this.css.pixelTop;
	}

	if (this.css)
	{
		// initialize the layer
		if (kq)
		{
			this.mbr_node.mbr_relativeLayerObject = this;
		}
		else
		{
			this.css.mbr_relativeLayerObject = this;
		}

		// add a dependecy to the parent layer if this is an embedded layer
		var var_parentlayer = this.getParentLayer();
		if (var_parentlayer)
		{
			this.addDependency(var_parentlayer.getId());
		}

		// notify event listeners
		this.fireEvent("init");
	}
};

RelativeLayer.prototype.deferCommand = function(par_command)
{
	this.mbr_deferredcommands[this.mbr_deferredcommands.length] = par_command;
};

RelativeLayer.prototype.processDeferredCommands = function()
{
	if (this.css)
	{
		// process the commands that could only be executed with a fully initialized relative layers object
		var var_index;

		for (var_index = 0; var_index < this.mbr_deferredcommands.length; var_index++)
		{
			eval(this.mbr_deferredcommands[var_index]);
		}

		// clear the deferred commands
		this.mbr_deferredcommands = null;

		// this is a very dirty hack to make drag and drop work under netscape 4.03, 4.04 and 4.05
		var_index = this.getId();
	}
};

RelativeLayer.prototype.setId = function(par_id)
{
	this.mbr_id = par_id;
};

RelativeLayer.prototype.getId = function()
{
	return this.mbr_id;
};

RelativeLayer.prototype.setZIndex = function(par_zindex)
{
	this.mbr_zIndex = par_zindex;
	this.mbr_zIndexChanged = true;
	this.refresh();
};

RelativeLayer.prototype.getZIndex = function()
{
	return this.css.zIndex;
};

RelativeLayer.prototype.getAbsoluteX = function()
{
	var var_x = this.getAbsoluteX();
	
	if (op || kq)
	{
		// since clip isn't supported, there's no way to get the visible x
	}
	else if (mz || ie || kq)
	{
		var var_cliparray = splitRect(this.css.clip);
		var_x += parseInt(var_cliparray[4]);
	}
	else if (ns)
	{
		var_x += this.css.clip.left;
	}
	
	return var_x;
};

RelativeLayer.prototype.getAbsoluteY = function()
{
	var var_y = this.getAbsoluteY();
	
	if (op || kq)
	{
		// since clip isn't supported, there's no way to get the visible y
	}
	else if (mz || ie)
	{
		var var_cliparray = splitRect(this.css.clip);
		var_y += parseInt(var_cliparray[1]);
	}
	else if (ns)
	{
		var_y += this.css.clip.top;
	}
	
	return var_y;
};

RelativeLayer.prototype.getAbsoluteWidth = function()
{
	var var_width = 0;

	if (op || kq)
	{
		// since clip isn't supported, there's no way to get the visible width
		var_width = this.css.pixelWidth;
	}
	else if (mz || ie || kq)
	{
		var var_cliparray = splitRect(this.css.clip);
		var_width = var_cliparray[2]-var_cliparray[4];
	}
	else if (ns)
	{
		var_width = this.css.clip.width;
	}

	return var_width;
};

RelativeLayer.prototype.getAbsoluteHeight = function()
{
	var var_height = 0;

	if (op || kq)
	{
		// since clip isn't supported, there's no way to get the visible height
		var_height = this.css.pixelHeight;
	}
	else if (mz || ie)
	{
		var var_cliparray = splitRect(this.css.clip);
		var_height = var_cliparray[3]-var_cliparray[1];
	}
	else if (ns)
	{
		var_height = this.css.clip.height;
	}

	return var_height;
};

RelativeLayer.prototype.isVisible = function()
{
	return this.mbr_visible;
};

RelativeLayer.prototype.setVisible = function(par_visible)
{
	this.mbr_visible = par_visible;
	this.refresh();
};

RelativeLayer.prototype.getBgColor = function()
{
	return this.mbr_bgColor;
};

RelativeLayer.prototype.setBgColor = function(par_bgColor)
{
	this.mbr_bgColor = par_bgColor;
	this.mbr_bgColorChanged = true;
	this.refresh();
};

RelativeLayer.prototype.getBgImage = function()
{
	return this.mbr_bgImage;
};

RelativeLayer.prototype.setBgImage = function(par_bgImage)
{
	this.mbr_bgImage = par_bgImage;
	this.mbr_bgImageChanged = true;
	this.refresh();
};

RelativeLayer.prototype.setHtml = function(par_text)
{
	if (this.css)
	{
		if (op)
		{
			// not yet possible with opera
			unsupportedBrowser();
		}
		else if (mz || ie || kq)
		{
			this.mbr_node.innerHTML = par_text;
		}
		else if (ns)
		{
			this.mbr_sethtml = null;
			this.css.document.open();
			this.css.document.write("<TABLE WIDTH=\""+this.calculateAvailableWidth()+"\" CELLPADDING=\"0\" CELLSPACING=\"0\" BORDER=\"0\"><TR><TD>"+par_text+"</TD></TR></TABLE>");
			this.css.document.close();
		}

		this.setPositioned(false);
		this.position();

		if (ns)
		{
			this.mbr_sethtml = par_text;
		}
	}
	else
	{
		this.deferCommand("this.setHtml(\""+par_text+"\")");
	}
};

RelativeLayer.prototype.raise = function()
{
	if (this.css)
	{
		this.setZIndex(this.getHighestZIndexInParent() + 1);
	
		// notify event listeners
		this.fireEvent("raise");
	}
	else
	{
		this.deferCommand("this.raise()");
	}
};

RelativeLayer.prototype.getHighestZIndexInParent = function()
{
	var var_highestzindex = 0;
	var var_layerid = 0;
	
	if (op)
	{
		for (var_layerid = 0; var_layerid < this.mbr_node.parentNode.getElementsByTagName("DIV").length; var_layerid++)
		{
			if (this.mbr_node.parentNode.getElementsByTagName("DIV")[var_layerid].style.zIndex > var_highestzindex)
			{
				var_highestzindex = this.mbr_node.parentNode.getElementsByTagName("DIV")[var_layerid].style.zIndex; 
			}
		}
	}
	else if (mz || kq)
    {
		for (var_layerid = 0; var_layerid < this.mbr_node.parentNode.childNodes.length; var_layerid++)
		{
			if (this.mbr_node.parentNode.childNodes[var_layerid].style != undefined &&
			   this.mbr_node.parentNode.childNodes[var_layerid].style.zIndex > var_highestzindex)
			{
				var_highestzindex = this.mbr_node.parentNode.childNodes[var_layerid].style.zIndex; 
			}
		}
    }
	else if (ns)
	{
		for (var_layerid = 0; var_layerid < this.css.parentLayer.document.layers.length; var_layerid++)
		{
			if (this.css.parentLayer.document.layers[var_layerid].zIndex > var_highestzindex)
			{
				var_highestzindex = this.css.parentLayer.document.layers[var_layerid].zIndex; 
			}
		}
	}
	else if (ie)
	{
		for (var_layerid = 0; var_layerid < this.mbr_node.parentElement.children.length; var_layerid++)
		{
			if (this.mbr_node.parentElement.children[var_layerid].style.zIndex > var_highestzindex)
			{
				var_highestzindex = this.mbr_node.parentElement.children[var_layerid].style.zIndex; 
			}
		}
	}
	
	return var_highestzindex;
};

RelativeLayer.prototype.intersects = function(par_layer)
{
	var var_x1 = this.getAbsoluteX();
	var var_y1 = this.getAbsoluteY();
	var var_width1 = this.getAbsoluteWidth();
	var var_height1 = this.getAbsoluteHeight();
	var var_x2 = par_layer.getAbsoluteX();
	var var_y2 = par_layer.getAbsoluteY();
	var var_width2 = par_layer.getAbsoluteWidth();
	var var_height2 = par_layer.getAbsoluteHeight();

	return (var_x2 + var_width2 > var_x1 &&
			var_y2 + var_height2 > var_y1 &&
			var_x2 < var_x1 + var_width1 &&
			var_y2 < var_y1 + var_height1);
};

RelativeLayer.prototype.addEventListener = function(par_type, par_listener)
{
	par_type = par_type.toLowerCase();

	if (!this.mbr_eventlisteners)
	{
		this.mbr_eventlisteners = new Hashtable();
	}

	var var_typelisteners = this.mbr_eventlisteners.get(par_type);
	if (!var_typelisteners)
	{
		var_typelisteners = new Array();
		this.mbr_eventlisteners.put(par_type, var_typelisteners);
	}
	
	switch(par_type)
	{
		case "click":
		case "mousedown":
		case "mousemove":
		case "mouseover":
		case "mouseup":
		case "keydown":
		case "keypress":
		case "keyup":
			this.addSystemEventListener(par_type);
			break;
	}
	var var_index = this.indexOfEventListener(par_type, par_listener);
	if (var_index == -1)
	{
		var_typelisteners.push(par_listener);
	}
};

RelativeLayer.prototype.removeEventListener = function(par_type, par_listener)
{
	par_type = par_type.toLowerCase();

	if (this.mbr_eventlisteners)
	{
		var var_typelisteners = this.mbr_eventlisteners.get(par_type);
		if (var_typelisteners)
		{
			switch(par_type)
			{
				case "click":
				case "mousedown":
				case "mousemove":
				case "mouseover":
				case "mouseup":
				case "keydown":
				case "keypress":
				case "keyup":
					this.removeSystemEventListener(par_type);
					break;
			}
			var_typelisteners.remove(par_listener);
		}
	}
};

RelativeLayer.prototype.fireEventObject = function(par_event)
{
	if (this.mbr_eventlisteners &&
	   this.mbr_eventlisteners.size() > 0)
	{
		var var_typelisteners = this.mbr_eventlisteners.get(par_event.type.toLowerCase());
		if (var_typelisteners)
		{
			var var_index;
			for (var_index = 0; var_index < var_typelisteners.length; var_index++)
			{
				var_typelisteners[var_index](par_event);
			}
		}
	}
};

RelativeLayer.prototype.makeEventObject = function(par_arguments)
{
	var var_eventobject = new Object();
	var_eventobject["type"] = par_arguments[0];
	var_eventobject["target"] = this;
	for (i = 1; i < par_arguments.length; i+=2)
	{
		var_eventobject[par_arguments[i]] = par_arguments[i+1];
	}
	return var_eventobject;
};

RelativeLayer.prototype.fireEvent = function()
{
	var var_eventobject = this.makeEventObject(arguments);
	this.fireEventObject(var_eventobject);
};

RelativeLayer.prototype.indexOfEventListener = function(par_type, par_listener)
{
	var var_result = -1;

	par_type = par_type.toLowerCase();
	
	var var_index;
	if (this.mbr_eventlisteners)
	{
		var var_typelisteners = this.mbr_eventlisteners.get(par_type);
		if (var_typelisteners)
		{
			var_result = var_typelisteners.indexOf(par_listener);
		}
	}

	return var_result;
};

RelativeLayer.prototype.systemEventListener = function(par_event)
{
	var var_event = normalizeEvent(par_event, this);
	var var_return = true;

	if (var_event &&
	   var_event.target)
	{
		var_event.target.fireEventObject(var_event);
		if (!op && !ns)
		{
			var_return = false;
		}
	}

	return var_return;
};

function normalizeEvent(par_event, par_currentobject)
{
	var var_event = new Object();
	var var_type = null;
	var var_target = null;

	if (op || mz || ns || kq)
	{
		var_type = par_event.type;
	}
	else if (ie)
	{
		var_type = window.event.type;
	}
	
	if (var_type)
	{
		if (op)
		{
			if (par_event.currentTarget.style &&
			   par_event.currentTarget.style.mbr_relativeLayerObject)
			{
				var_target = par_event.currentTarget.style.mbr_relativeLayerObject;
			}
			// cancel the event bubbling
			par_event.cancelBubble = true;
			par_event.stopPropagation();
		}
		else if (mz)
		{
			if (par_event.currentTarget.style &&
			   par_event.currentTarget.style.mbr_relativeLayerObject)
			{
				var_target = par_event.currentTarget.style.mbr_relativeLayerObject;
			}
			// cancel the event bubbling
			par_event.cancelBubble = true;
			par_event.stopPropagation();
		}
		else if (kq)
		{
			if (par_event.currentTarget &&
			   par_event.currentTarget.mbr_relativeLayerObject)
			{
				var_target = par_event.currentTarget.mbr_relativeLayerObject;
			}
			// cancel the event bubbling
			par_event.cancelBubble = true;
			par_event.stopPropagation();
		}
		else if (ns)
		{
			var route = routeEvent(par_event);
			if (!(route == false) && route != true)
			{
				if (par_currentobject)
				{
					if (par_currentobject.mbr_relativeLayerObject)
					{
						var_target = par_currentobject.mbr_relativeLayerObject;
					}
					// if not, use a dirty hack to obtain the relative layer through its id string
					else if (par_currentobject.var_index)
					{
						var_target = getRelativeLayer(par_currentobject.var_index);
					}
				}
			}
			else
			{
				var_event = null;
			}
		}
		else if (ie)
		{
			if (par_currentobject &&
			   par_currentobject.style &&
			   par_currentobject.style.mbr_relativeLayerObject)
			{
				var_target = par_currentobject.style.mbr_relativeLayerObject;
			}
			window.event.cancelBubble = true;
		}
	}

	if (var_event)
	{
		var_event.target = var_target;
		var_event.type = var_type;

		switch(var_type)
		{
			case "keydown":
			case "keypress":
			case "keyup":
				break;
			case "click":
			case "mousedown":
			case "mouseup":
				if (op || mz || ns || kq)
				{
					var_event.which = par_event.which;
				}
				else if (ie)
				{
					var_event.which = window.event.button;
				}
			case "mouseover":
			case "mousemove":
				if (op)
				{
					var_event.pageX = par_event.clientX;
					var_event.pageY = par_event.clientY;
					if (var_target)
					{
						var_event.layerX = var_event.pageX-var_target.getAbsolutePageX();
						var_event.layerY = var_event.pageY-var_target.getAbsolutePageY();
					}
				}
				else if (mz)
				{
					var_event.pageX = par_event.pageX;
					var_event.pageY = par_event.pageY;
					if (var_target)
					{
						var_event.layerX = par_event.layerX;
						var_event.layerY = par_event.layerY;
					}
				}
				else if (kq)
				{
					var_event.pageX = par_event.clientX;
					var_event.pageY = par_event.clientY;
					if (var_target)
					{
						var_event.layerX = var_event.pageX-var_target.getAbsolutePageX();
						var_event.layerY = var_event.pageY-var_target.getAbsolutePageY();
					}
				}
				else if (ns)
				{
					var_event.pageX = par_event.pageX;
					var_event.pageY = par_event.pageY;
					if (var_target)
					{
						var_event.layerX = par_event.layerX;
						var_event.layerY = par_event.layerY;
					}
				}
				else if (ie)
				{
					var_event.pageX = window.event.clientX+document.body.scrollLeft;
					var_event.pageY = window.event.clientY+document.body.scrollTop;
					if (var_target)
					{
						var_event.layerX = var_event.pageX-var_target.getAbsolutePageX();
						var_event.layerY = var_event.pageY-var_target.getAbsolutePageY();
					}
				}
				break;
		}
	}

	return var_event;
};

RelativeLayer.prototype.addSystemEventListener = function(par_type)
{
	if (this.css)
	{
		if (op)
		{
			this.mbr_node["on"+par_type] = this.systemEventListener;
		}
		else if (mz || kq)
		{
			this.mbr_node.addEventListener(par_type, this.systemEventListener, false);
		}
		else if (ns)
		{
			this.css["on"+par_type] = this.systemEventListener;
			this.css.captureEvents(Event[par_type.toUpperCase()]);
		}
		else if (ie)
		{
			this.mbr_node["on"+par_type] = this.systemEventListener;
		}
	}
	else
	{
		this.deferCommand("this.addSystemEventListener(\""+par_type+"\","+this.systemEventListener+")");
	}
};

RelativeLayer.prototype.removeSystemEventListener = function(par_type)
{
	if (this.css)
	{
		if (op)
		{
			this.mbr_node["on"+par_type] = null;
		}
		else if (mz || kq)
		{
			this.mbr_node.removeEventListener(par_type, this.systemEventListener, false);
		}
		else if (ns)
		{
			this.css["on"+par_type] = null;
			this.css.releaseEvents(Event[par_type.toUpperCase()]);
		}
		else if (ie)
		{
			this.mbr_node["on"+par_type] = null;
		}
	}
	else
	{
		this.deferCommand("this.removeSystemEventListener(\""+par_type+"\","+this.systemEventListener+")");
	}
};

RelativeLayer.prototype.defineVisibleArea = function(
	par_visibleHorizOffsetReference, par_visibleVertOffsetReference,
	par_visibleWidth, par_visibleHeight,
	par_visibleHorizBorder, par_visibleHorizOffset, par_visibleVertBorder, par_visibleVertOffset,
	par_visibleHandleHorizOffset, par_visibleHandleVertOffset)
{
	this.defineVisibleHorizOffsetReference(par_visibleHorizOffsetReference);  
	this.defineVisibleVertOffsetReference(par_visibleVertOffsetReference);  
	this.defineVisibleWidth(par_visibleWidth);  
	this.defineVisibleHeight(par_visibleHeight);
	this.defineVisibleHandleHorizOffset(par_visibleHandleHorizOffset);
	this.defineVisibleHandleVertOffset(par_visibleHandleVertOffset);
	this.defineVisibleHorizBorder(par_visibleHorizBorder);	
	this.defineVisibleHorizOffset(par_visibleHorizOffset);
	this.defineVisibleVertBorder(par_visibleVertBorder);  
	this.defineVisibleVertOffset(par_visibleVertOffset);
};

RelativeLayer.prototype.defineVisibleHorizOffsetReference = function(par_visibleHorizOffsetReference)
{
	var var_index;

	if (this.mbr_visibleHorizOffsetReferences &&
	   this.mbr_visibleHorizOffsetReferences.length > 0)
	{
		for (var_index = 0; var_index < this.mbr_visibleHorizOffsetReferences.length; var_index++)
		{
			this.removeDependency(this.mbr_visibleHorizOffsetReferences[var_index]);
		}
	}

	this.mbr_visibleHorizOffsetReferences = new Array();
	
	if (par_visibleHorizOffsetReference != "")
	{
		this.mbr_visibleHorizOffsetReferences = par_visibleHorizOffsetReference.split(/\s*\,\s*/);

		if (this.mbr_visibleHorizOffsetReferences.length > 0)
		{
			for (var_index = 0; var_index < this.mbr_visibleHorizOffsetReferences.length; var_index++)
			{
				this.addDependency(this.mbr_visibleHorizOffsetReferences[var_index]);
			}
		}
	}
};

RelativeLayer.prototype.defineVisibleVertOffsetReference = function(par_visibleVertOffsetReference)
{
	var var_index;

	if (this.mbr_visibleVertOffsetReferences &&
	   this.mbr_visibleVertOffsetReferences.length > 0)
	{
		for (var_index = 0; var_index < this.mbr_visibleVertOffsetReferences.length; var_index++)
		{
			this.removeDependency(this.mbr_visibleVertOffsetReferences[var_index]);
		}
	}

	this.mbr_visibleVertOffsetReferences = new Array();
	
	if (par_visibleVertOffsetReference != "")
	{
		this.mbr_visibleVertOffsetReferences = String(par_visibleVertOffsetReference).split(/\s*\,\s*/);
		
		if (this.mbr_visibleVertOffsetReferences.length > 0)
		{
			for (var_index = 0; var_index < this.mbr_visibleVertOffsetReferences.length; var_index++)
			{
				this.addDependency(this.mbr_visibleVertOffsetReferences[var_index]);
			}
		}
	}
};

RelativeLayer.prototype.defineVisibleWidth = function(par_visibleWidth)
{
	if (par_visibleWidth == "AUTO")
	{
		this.mbr_autoVisibleWidth = true;
	}
	else
	{
		this.mbr_visibleWidth = par_visibleWidth;
	}
};

RelativeLayer.prototype.defineVisibleHeight = function(par_visibleHeight)
{
	if (par_visibleHeight == "AUTO")
	{
		this.mbr_autoVisibleHeight = true;
	}
	else
	{
		this.mbr_visibleHeight = par_visibleHeight;
	}
};

RelativeLayer.prototype.updateAutoVisibleWidth = function()
{
	if (this.mbr_autoVisibleWidth)
	{
		if (op)
		{
			this.mbr_visibleWidth = parseInt(this.css.pixelWidth);
		}
		else if (mz || kq)
		{
			this.mbr_visibleWidth = parseInt(this.mbr_node.offsetWidth);
		}
		else if (ns)
		{
			this.mbr_visibleWidth = parseInt(this.css.document.width);
		}
		else if (ie)
		{
			this.mbr_visibleWidth = parseInt(this.mbr_node.offsetWidth);
		}
	}
};

RelativeLayer.prototype.updateAutoVisibleHeight = function()
{
	if (this.mbr_autoVisibleHeight)
	{
		if (op)
		{
			this.mbr_visibleHeight = parseInt(this.css.pixelHeight);
		}
		else if (mz || ie || kq)
		{
			this.mbr_visibleHeight = parseInt(this.mbr_node.offsetHeight);
		}
		else if (ns)
		{
			this.mbr_visibleHeight = parseInt(this.css.document.height);
		}
	}
};

RelativeLayer.prototype.defineVisibleHandleHorizOffset = function(par_visibleHandleHorizOffset)
{
	this.mbr_visibleHandleHorizOffset = par_visibleHandleHorizOffset;  
};

RelativeLayer.prototype.defineVisibleHandleVertOffset = function(par_visibleHandleVertOffset)
{
	this.mbr_visibleHandleVertOffset = par_visibleHandleVertOffset;  
};

RelativeLayer.prototype.defineVisibleHorizBorder = function(par_visibleHorizBorder)
{
	this.mbr_visibleHorizBorder = par_visibleHorizBorder;  
};

RelativeLayer.prototype.defineVisibleHorizOffset = function(par_visibleHorizOffset)
{
	this.mbr_visibleHorizOffset = par_visibleHorizOffset;  
};

RelativeLayer.prototype.defineVisibleVertBorder = function(par_visibleVertBorder)
{
	this.mbr_visibleVertBorder = par_visibleVertBorder;  
};

RelativeLayer.prototype.defineVisibleVertOffset = function(par_visibleVertOffset)
{
	this.mbr_visibleVertOffset = par_visibleVertOffset;  
};

RelativeLayer.prototype.defineAvailableArea = function(
	par_availableWidth, par_availableHeight,
	par_availableHorizBorder, par_availableHorizOffset, par_availableVertBorder, par_availableVertOffset,
	par_availableHandleHorizOffset, par_availableHandleVertOffset)
{
	this.defineAvailableWidth(par_availableWidth);  
	this.defineAvailableHeight(par_availableHeight);
	this.defineAvailableHandleHorizOffset(par_availableHandleHorizOffset);
	this.defineAvailableHandleVertOffset(par_availableHandleVertOffset);
	this.defineAvailableHorizBorder(par_availableHorizBorder);	
	this.defineAvailableHorizOffset(par_availableHorizOffset);
	this.defineAvailableVertBorder(par_availableVertBorder);  
	this.defineAvailableVertOffset(par_availableVertOffset);
};

RelativeLayer.prototype.defineAvailableWidth = function(par_availableWidth)
{
	if (par_availableWidth == "AUTO")
	{
		this.mbr_autoAvailableWidth = true;
	}
	else
	{
		this.mbr_autoAvailableWidth = false;
		this.mbr_availableWidth = par_availableWidth;
	}
};

RelativeLayer.prototype.defineAvailableHeight = function(par_availableHeight)
{
	if (par_availableHeight == "AUTO")
	{
		this.mbr_autoAvailableHeight = true;
	}
	else
	{
		this.mbr_autoAvailableHeight = false;
		this.mbr_availableHeight = par_availableHeight;
	}
};

RelativeLayer.prototype.updateAutoAvailableWidth = function()
{
	if (this.mbr_autoAvailableWidth)
	{
		if (op)
		{
			this.mbr_availableWidth = parseInt(this.css.pixelWidth);
		}
		else if (mz || ie || kq)
		{
			this.mbr_availableWidth = parseInt(this.mbr_node.offsetWidth);
		}
		else if (ns)
		{
			this.mbr_availableWidth = parseInt(this.css.document.width);
		}
	}
};

RelativeLayer.prototype.updateAutoAvailableHeight = function()
{
	if (this.mbr_autoAvailableHeight)
	{
		if (op)
		{
			this.mbr_availableHeight = parseInt(this.css.pixelHeight);
		}
		else if (mz || ie || kq)
		{
			this.mbr_availableHeight = parseInt(this.mbr_node.offsetHeight);
		}
		else if (ns)
		{
			this.mbr_availableHeight = parseInt(this.css.document.height);
		}
	}
};

RelativeLayer.prototype.defineAvailableHandleHorizOffset = function(par_availableHandleHorizOffset)
{
	this.mbr_availableHandleHorizOffset = par_availableHandleHorizOffset;  
};

RelativeLayer.prototype.defineAvailableHandleVertOffset = function(par_availableHandleVertOffset)
{
	this.mbr_availableHandleVertOffset = par_availableHandleVertOffset;  
};

RelativeLayer.prototype.defineAvailableHorizBorder = function(par_availableHorizBorder)
{
	this.mbr_availableHorizBorder = par_availableHorizBorder;  
};

RelativeLayer.prototype.defineAvailableHorizOffset = function(par_availableHorizOffset)
{
	this.mbr_availableHorizOffset = par_availableHorizOffset;
};

RelativeLayer.prototype.defineAvailableVertBorder = function(par_availableVertBorder)
{
	this.mbr_availableVertBorder = par_availableVertBorder;  
};

RelativeLayer.prototype.defineAvailableVertOffset = function(par_availableVertOffset)
{
	this.mbr_availableVertOffset = par_availableVertOffset;
};

RelativeLayer.prototype.getVisibleHorizOffsetReferenceId = function()
{
	return this.mbr_visibleHorizOffsetReferenceId;
};

RelativeLayer.prototype.getVisibleVertOffsetReferenceId = function()
{
	return this.mbr_visibleVertOffsetReferenceId;
};

RelativeLayer.prototype.getVisibleWidth = function()
{
	return this.mbr_visibleWidth;
};

RelativeLayer.prototype.getVisibleHeight = function()
{
	return this.mbr_visibleHeight;
};

RelativeLayer.prototype.getVisibleHorizBorder = function()
{
	return this.mbr_visibleHorizBorder;
};

RelativeLayer.prototype.getVisibleHorizOffset = function()
{
	return this.mbr_visibleHorizOffset;
};

RelativeLayer.prototype.getVisibleVertBorder = function()
{
	return this.mbr_visibleVertBorder;
};

RelativeLayer.prototype.getVisibleVertOffset = function()
{
	return this.mbr_visibleVertOffset;
};

RelativeLayer.prototype.getVisibleHandleHorizOffset = function()
{
	return this.mbr_visibleHandleHorizOffset;
};

RelativeLayer.prototype.getVisibleHandleVertOffset = function()
{
	return this.mbr_visibleHandleVertOffset;
};

RelativeLayer.prototype.getAvailableWidth = function()
{
	return this.mbr_availableWidth;
};

RelativeLayer.prototype.getAvailableHeight = function()
{
	return this.mbr_availableHeight;
};

RelativeLayer.prototype.getAvailableHorizBorder = function()
{
	return this.mbr_availableHorizBorder;
};

RelativeLayer.prototype.getAvailableHorizOffset = function()
{
	return this.mbr_availableHorizOffset;
};

RelativeLayer.prototype.getAvailableVertBorder = function()
{
	return this.mbr_availableVertBorder;
};

RelativeLayer.prototype.getAvailableVertOffset = function()
{
	return this.mbr_availableVertOffset;
};

RelativeLayer.prototype.getAvailableHandleHorizOffset = function()
{
	return this.mbr_availableHandleHorizOffset;
};

RelativeLayer.prototype.getAvailableHandleVertOffset = function()
{
	return this.mbr_availableHandleVertOffset;
};

RelativeLayer.prototype.getParentLayer = function()
{
	var var_parentlayer = null;

	if (op || mz)
	{
		if (this.mbr_node &&
		   this.mbr_node.parentNode &&
		   this.mbr_node.parentNode.style.mbr_relativeLayerObject)
		{
			var_parentlayer = this.mbr_node.parentNode.style.mbr_relativeLayerObject;
		}
	}
	else if (ns)
	{
		if (this.css &&
		   this.css.parentLayer &&
		   this.css.parentLayer.mbr_relativeLayerObject)
		{
			var_parentlayer = this.css.parentLayer.mbr_relativeLayerObject;
		}
	}
	else if (kq)
	{
		if (this.mbr_node &&
		   this.mbr_node.parentNode &&
		   this.mbr_node.parentNode.mbr_relativeLayerObject)
		{
			var_parentlayer = this.mbr_node.parentNode.mbr_relativeLayerObject;
		}
	}
	else if (ie)
	{
		if (this.mbr_node &&
		   this.mbr_node.parentElement &&
		   this.mbr_node.parentElement.style.mbr_relativeLayerObject)
		{
			var_parentlayer = this.mbr_node.parentElement.style.mbr_relativeLayerObject;
		}
	}

	return var_parentlayer;
};

RelativeLayer.prototype.getAbsoluteCenterX = function()
{
	var var_visiblewidth = this.calculateVisibleWidth();
	var var_centerx;
	
	// get the absolute position of the visible center relative to the parent layer
	if (matchPercentage(String(this.mbr_visibleHandleHorizOffset)))
	{
		var_centerx = Math.floor(this.getAbsoluteX() + (var_visiblewidth / 2) + ((parseFloat(this.mbr_visibleHandleHorizOffset.toString().chop()) * var_visiblewidth) / 100));
	}
	else
	{
		var_centerx = Math.floor(this.getAbsoluteX() + (var_visiblewidth / 2) + parseInt(this.mbr_visibleHandleHorizOffset));
	}
	
	// calculate the offset of the parent layers
	var var_parentlayer = this.getParentLayer();
	if (var_parentlayer)
	{
		var_centerx += var_parentlayer.getAbsolutePageX();
	}
	
	return var_centerx;
};

RelativeLayer.prototype.getAbsoluteCenterY = function()
{
	var var_visiblewidth = this.calculateVisibleHeight();
	var var_centery;
	
	// get the absolute position of the visible center relative to the parent layer
	if (matchPercentage(String(this.mbr_visibleHandleVertOffset)))
	{
		var_centery = this.getAbsoluteY() + (var_visiblewidth / 2) + ((parseFloat(this.mbr_visibleHandleVertOffset.toString().chop()) * var_visiblewidth) / 100);
	}
	else
	{
		var_centery = this.getAbsoluteY() + (var_visiblewidth / 2) + parseInt(this.mbr_visibleHandleVertOffset);
	}
	
	// calculate the offset of the parent layers
	var var_parentlayer = this.getParentLayer();
	if (var_parentlayer)
	{
		var_centery += var_parentlayer.getAbsolutePageY();
	}
	
	return var_centery;
};

RelativeLayer.prototype.getParentWidth = function()
{
	var var_parentwidth;
	var var_parentlayer = this.getParentLayer();

	if (var_parentlayer)
	{
		var_parentwidth = var_parentlayer.calculateAvailableWidth();
	}
	else
	{
		var_parentwidth = getWindowWidth();
	}

	return var_parentwidth;
};

RelativeLayer.prototype.getParentHeight = function()
{
	var var_parentheight;
	var var_parentlayer = this.getParentLayer();

	if (var_parentlayer)
	{
		var_parentheight = var_parentlayer.calculateAvailableHeight();
	}
	else
	{
		var_parentheight = getWindowHeight();
	}

	return var_parentheight;
};

RelativeLayer.prototype.calculateVisibleWidth = function()
{
	var var_parentWidth;
	var var_visibleWidth;
	
	// check if the visible width is just digits without any operators nor percentages
	if (matchNumber(this.mbr_visibleWidth))
	{
		// only digits
		var_visibleWidth = parseInt(this.mbr_visibleWidth);
	}
	else
	{
		// get the parent width
		var_parentWidth = this.getParentWidth();

		// with operators
		// concanate the percentage and pixel string to exactly 2 parts, first the percentage, then the pixels
		var var_concanatedstring = concanatePercentagePixels(this.mbr_visibleWidth);
		// separate the 2 parts and isolate the operator
		var var_matches = matchPercentagesAndNumbers(var_concanatedstring);
		// calculate the percentage according to the parent and add the fixed pixels to it
		var_visibleWidth = Math.floor((var_parentWidth * parseFloat(var_matches[0]))/100) + parseInt(var_matches[1]);
	}

	return var_visibleWidth;
};

RelativeLayer.prototype.calculateVisibleHeight = function()
{
	var var_parentHeight;
	var var_visibleHeight;
	
	// check if the visible height is just digits without any operators nor percentages
	if (matchNumber(this.mbr_visibleHeight))
	{
		// only digits
		var_visibleHeight = parseInt(this.mbr_visibleHeight);
	}
	else
	{
		// get the parent height
		var_parentHeight = this.getParentHeight();

		// with operators
		// concanate the percentage and pixel string to exactly 2 parts, first the percentage, then the pixels
		var var_concanatedstring = concanatePercentagePixels(this.mbr_visibleHeight);
		// separate the 2 parts and isolate the operator
		var var_matches = matchPercentagesAndNumbers(var_concanatedstring);
		// calculate the percentage according to the parent and add the fixed pixels to it
		var_visibleHeight = Math.floor((var_parentHeight * parseFloat(var_matches[0]))/100) + parseInt(var_matches[1]);
	}
	
	return var_visibleHeight;
};

RelativeLayer.prototype.calculateVisibleHandleHorizOffset = function(par_visibleWidth)
{	
	var var_visiblewidth;
	var var_visiblecenterhorizoffset;
	
	// get the visible area width
	if (arguments.length < 1)
	{
		var_visiblewidth = this.calculateVisibleWidth();
	}
	else
	{
		var_visiblewidth = par_visibleWidth;
	}
	
	// check if the visible area horizontal offset is just digits without any operators nor percentages
	if (matchNumber(String(this.mbr_visibleHandleHorizOffset)))
	{
		var_visiblecenterhorizoffset = parseInt(this.mbr_visibleHandleHorizOffset) + Math.floor(var_visiblewidth/2);
	}
	else
	{
		// with operators
		// concanate the percentage and pixel string to exactly 2 parts, first the percentage, then the pixels
		var var_concanatedstring = concanatePercentagePixels(this.mbr_visibleHandleHorizOffset);
		// separate the 2 parts and isolate the operator
		var var_matches = matchPercentagesAndNumbers(var_concanatedstring);
		var var_percentage = parseFloat(var_matches[0]);
		var var_number = parseInt(var_matches[1]);
		// prevent rounding errors if the positioning is meant to be exactly the left or right edge
		if (var_percentage == -50 && Math.abs(var_number) == 0)
		{
			var_visiblecenterhorizoffset = 0;
		}
		else if (var_percentage == 50 && Math.abs(var_number) == 0)
		{
			var_visiblecenterhorizoffset = parseInt(var_visiblewidth);
		}
		else
		{
			var_visiblecenterhorizoffset = Math.floor((var_visiblewidth * var_percentage/100)) + var_number + Math.floor(var_visiblewidth/2);
		}
	}
	
	return var_visiblecenterhorizoffset;
};
	
RelativeLayer.prototype.calculateVisibleHandleVertOffset = function(par_visibleHeight)
{	
	var var_visibleheight;
	var var_visiblecentervertoffset;
	
	// get the visible area height
	if (arguments.length < 1)
	{
		var_visibleheight = this.calculateVisibleHeight();
	}
	else
	{
		var_visibleheight = par_visibleHeight;
	}
	
	
	// check if the visible area horizontal offset is just digits without any operators nor percentages
	if (matchNumber(String(this.mbr_visibleHandleVertOffset)))
	{
		var_visiblecentervertoffset = parseInt(this.mbr_visibleHandleVertOffset) + Math.floor(var_visibleheight/2);
	}
	else
	{
		// with operators
		// concanate the percentage and pixel string to exactly 2 parts, first the percentage, then the pixels
		var var_concanatedstring = concanatePercentagePixels(this.mbr_visibleHandleVertOffset);
		// separate the 2 parts and isolate the operator
		var var_matches = matchPercentagesAndNumbers(var_concanatedstring);
		var var_percentage = parseFloat(var_matches[0]);
		var var_number = parseInt(var_matches[1]);
		// prevent rounding errors if the positioning is meant to be exactly the left or right edge
		if (var_percentage == -50 && Math.abs(var_number) == 0)
		{
			var_visiblecentervertoffset = 0;
		}
		else if (var_percentage == 50 && Math.abs(var_number) == 0)
		{
			var_visiblecentervertoffset = parseInt(var_visibleheight);
		}
		else
		{
			var_visiblecentervertoffset = Math.floor((var_visibleheight * var_percentage/100)) + var_number + Math.floor(var_visibleheight/2);
		}
	}
	
	return var_visiblecentervertoffset;
};

RelativeLayer.prototype.calculateVisibleHorizOffset = function(par_visibleWidth)
{
	var var_parentwidth;
	var var_visiblewidth;
	var var_visiblehorizoffset;
	var var_visiblecenterhorizoffset;

	// get the parent width
	var_parentwidth = this.getParentWidth();
	
	// get the visible area width
	if (arguments.length < 1)
	{
		var_visiblewidth = this.calculateVisibleWidth();
	}
	else
	{
		var_visiblewidth = par_visibleWidth;
	}

	// get the visible area center horizontal offset
	var_visiblecenterhorizoffset = this.calculateVisibleHandleHorizOffset(var_visiblewidth);
	
	// check if the visible area horizontal offset is just digits without any operators nor percentages
	if (matchNumber(String(this.mbr_visibleHorizOffset)))
	{
		// only digits
		var_visiblehorizoffset = parseInt(this.mbr_visibleHorizOffset);
	}
	else
	{
		// with operators
		// concanate the percentage and pixel string to exactly 2 parts, first the percentage, then the pixels
		var var_concanatedstring = concanatePercentagePixels(this.mbr_visibleHorizOffset);
		// separate the 2 parts and isolate the operator
		var var_matches = matchPercentagesAndNumbers(var_concanatedstring);
		// calculate the percentage according to the parent and add the fixed pixels to it
		var_visiblehorizoffset = Math.floor((var_parentwidth * parseFloat(var_matches[0]))/100) + parseInt(var_matches[1]);
	}
	
	// check if the vertical offset is defined relative to the parent layer or to another reference layer
	if (this.mbr_visibleHorizOffsetReferences.length == 0)
	{
		// calcute the absolute visible area vertical offset relative to the parent border
		if (this.mbr_visibleHorizBorder == RIGHT)
		{
			var_visiblehorizoffset = var_parentwidth + var_visiblehorizoffset;
		}
	}
	else
	{
		// offset is relative to another layer
		if (this.mbr_visibleHorizBorder == LEFT)
		{
			var var_referencelayer;
			var var_referencelayer_visiblehorizoffset;
			var var_referencelayer_rightmost_visibleleftedge;
			
			// get the rightmost leftedge
			for (var var_referencelayer_index = 0; var_referencelayer_index < this.mbr_visibleHorizOffsetReferences.length; var_referencelayer_index++)
			{
				var_referencelayer = getRelativeLayer(this.mbr_visibleHorizOffsetReferences[var_referencelayer_index]);
				var_referencelayer_visiblehorizoffset = var_referencelayer.calculateVisibleHorizOffset();
				if (var_referencelayer_index == 0 ||
				   var_referencelayer_rightmost_visibleleftedge < var_referencelayer_visiblehorizoffset)
				{
					var_referencelayer_rightmost_visibleleftedge = var_referencelayer_visiblehorizoffset;
				}
			}
		
			var_visiblehorizoffset += var_referencelayer_rightmost_visibleleftedge;
		}
		else
		{
			var var_referencelayer;
			var var_referencelayer_visiblerightedge;
			var var_referencelayer_rightmost_visiblerightedge;
			
			// get the rightmost rightedge
			for (var var_referencelayer_index = 0; var_referencelayer_index < this.mbr_visibleHorizOffsetReferences.length; var_referencelayer_index++)
			{
				var_referencelayer = getRelativeLayer(this.mbr_visibleHorizOffsetReferences[var_referencelayer_index]);
				var_referencelayer_visiblerightedge =  var_referencelayer.calculateVisibleHorizOffset()+var_referencelayer.calculateVisibleWidth();
				if (var_referencelayer_index == 0 ||
				   var_referencelayer_rightmost_visiblerightedge < var_referencelayer_visiblerightedge)
				{
					var_referencelayer_rightmost_visiblerightedge = var_referencelayer_visiblerightedge;
				}
			}
		
			var_visiblehorizoffset += var_referencelayer_rightmost_visiblerightedge;
		}
	}
	
	// adapt it to the visible center horizontal offset
	var_visiblehorizoffset = var_visiblehorizoffset - var_visiblecenterhorizoffset;
	
	return var_visiblehorizoffset;
};

RelativeLayer.prototype.calculateVisibleVertOffset = function(par_visibleHeight)
{
	var var_parentheight;
	var var_visibleheight;
	var var_visiblevertoffset;
	var var_visiblecentervertoffset;
	
	// get the parent height
	var_parentheight = this.getParentHeight();
	
	// get the visible area height
	if (arguments.length < 1)
	{
		var_visibleheight = this.calculateVisibleHeight();
	}
	else
	{
		var_visibleheight = par_visibleHeight;
	}

	// get the visible area center vertical offset
	var_visiblecentervertoffset = this.calculateVisibleHandleVertOffset(var_visibleheight);

	// check if the visible area vertical offset is just digits without any operators nor percentages
	if (matchNumber(String(this.mbr_visibleVertOffset)))
	{
		// only digits
		var_visiblevertoffset = parseInt(this.mbr_visibleVertOffset);
	}
	else
	{
		// with operators
		// concanate the percentage and pixel string to exactly 2 parts, first the percentage, then the pixels
		var var_concanatedstring = concanatePercentagePixels(this.mbr_visibleVertOffset);
		// separate the 2 parts and isolate the operator
		var var_matches = matchPercentagesAndNumbers(var_concanatedstring);
		// calculate the percentage according to the parent and add the fixed pixels to it
		var_visiblevertoffset = Math.floor((var_parentheight * parseFloat(var_matches[0]))/100) + parseInt(var_matches[1]);
	}
	
	// check if the vertical offset is defined relative to the parent layer or to another relative layer
	if (this.mbr_visibleVertOffsetReferences.length == 0)
	{
		// calcute the absolute visible area vertical offset relative to the parent border
		if (this.mbr_visibleVertBorder == BOTTOM)
		{
			var_visiblevertoffset = var_parentheight + var_visiblevertoffset;
		}
	}
	else
	{
		// offset is relative to another layer
		if (this.mbr_visibleVertBorder == TOP)
		{
			var var_referencelayer;
			var var_referencelayer_visiblevertoffset;
			var var_referencelayer_lowest_visibletopedge;
			
			// get the lowest topedge
			for (var var_referencelayer_index = 0; var_referencelayer_index < this.mbr_visibleVertOffsetReferences.length; var_referencelayer_index++)
			{
				var_referencelayer = getRelativeLayer(this.mbr_visibleVertOffsetReferences[var_referencelayer_index]);
				var_referencelayer_visiblevertoffset =  var_referencelayer.calculateVisibleVertOffset();
				if (var_referencelayer_index == 0 ||
				   var_referencelayer_lowest_visibletopedge < var_referencelayer_visiblevertoffset)
				{
					var_referencelayer_lowest_visibletopedge = var_referencelayer_visiblevertoffset;
				}
			}
		
			var_visiblevertoffset += var_referencelayer_lowest_visibletopedge;
		}
		else
		{
			var var_referencelayer;
			var var_referencelayer_visiblebottomedge;
			var var_referencelayer_lowest_visiblebottomedge;
			
			// get the lowest bottomedge
			for (var var_referencelayer_index = 0; var_referencelayer_index < this.mbr_visibleVertOffsetReferences.length; var_referencelayer_index++)
			{
				var_referencelayer = getRelativeLayer(this.mbr_visibleVertOffsetReferences[var_referencelayer_index]);
				var_referencelayer_visiblebottomedge =  var_referencelayer.calculateVisibleVertOffset()+var_referencelayer.calculateVisibleHeight();
				if (var_referencelayer_index == 0 ||
				   var_referencelayer_lowest_visiblebottomedge < var_referencelayer_visiblebottomedge)
				{
					var_referencelayer_lowest_visiblebottomedge = var_referencelayer_visiblebottomedge;
				}
			}
		
			var_visiblevertoffset += var_referencelayer_visiblebottomedge;
		}
	}
	
	// adapt it to the visible center vertical offset
	var_visiblevertoffset = var_visiblevertoffset - var_visiblecentervertoffset;

	return var_visiblevertoffset;
};

RelativeLayer.prototype.calculateAvailableWidth = function(par_visibleWidth)
{
	var var_visiblewidth;
	var var_availablewidth;
	
	// get the visible width
	if (arguments.length < 1)
	{
		var_visiblewidth = this.calculateVisibleWidth();
	}
	else
	{
		var_visiblewidth = par_visibleWidth;
	}

	// check if the available width is just digits without any operators nor percentages
	if (matchNumber(String(this.mbr_availableWidth)))
	{
		// only digits
		var_availablewidth = parseInt(this.mbr_availableWidth);
	}
	else
	{
		// with operators
		// concanate the percentage and pixel string to exactly 2 parts, first the percentage, then the pixels
		var var_concanatedstring = concanatePercentagePixels(this.mbr_availableWidth);
		// separate the 2 parts and isolate the operator
		var var_matches = matchPercentagesAndNumbers(var_concanatedstring);
		// calculate the percentage according to the visible area and add the fixed pixels to it
		var_availablewidth = Math.floor((var_visiblewidth * parseFloat(var_matches[0]))/100) + parseInt(var_matches[1]);
	}
	
	// limit the available width to be minimum the width of the visible area
	if (var_availablewidth < var_visiblewidth)
	{
		var_availablewidth = var_visiblewidth;
	}
	
	return var_availablewidth;
};

RelativeLayer.prototype.calculateAvailableHeight = function(par_visibleHeight)
{
	var var_visibleheight;
	var var_availableheight;
	
	// get the visible height
	if (arguments.length < 1)
	{
		var_visibleheight = this.calculateVisibleHeight();
	}
	else
	{
		var_visibleheight = par_visibleHeight;
	}
	
	// check if the available height is just digits without any operators nor percentages
	if (matchNumber(String(this.mbr_availableHeight)))
	{
		// only digits
		var_availableheight = parseInt(this.mbr_availableHeight);
	}
	else
	{
		// with operators
		// concanate the percentage and pixel string to exactly 2 parts, first the percentage, then the pixels
		var var_concanatedstring = concanatePercentagePixels(this.mbr_availableHeight);
		// separate the 2 parts and isolate the operator
		var var_matches = matchPercentagesAndNumbers(var_concanatedstring);
		// calculate the percentage according to the visible area and add the fixed pixels to it
		var_availableheight = Math.floor((var_visibleheight * parseFloat(var_matches[0]))/100) + parseInt(var_matches[1]);
	}
	
	// limit the available height to be minimum the height of the visible area
	if (var_availableheight < var_visibleheight)
	{
		var_availableheight = var_visibleheight;
	}
	
	return var_availableheight;
};

RelativeLayer.prototype.calculateAvailableHandleHorizOffset = function()
{	
	var var_availablewidth;
	var var_availablecenterhorizoffset;
	
	// get the available area width
	var_availablewidth = this.calculateAvailableWidth();
	
	// calculate the available area center horizontal offset
	if (matchPercentage(String(this.mbr_availableHandleHorizOffset)))
	{
		var_availablecenterhorizoffset = Math.floor((var_availablewidth * parseFloat(this.mbr_availableHandleHorizOffset.toString().chop()) )/100);
	}
	else
	{
		var_availablecenterhorizoffset = parseInt(this.mbr_availableHandleHorizOffset);
	}
	
	return var_availablecenterhorizoffset;
};
	
RelativeLayer.prototype.calculateAvailableHandleVertOffset = function()
{	
	var var_availableheight;
	var var_availablecentervertoffset;
	
	// get the available area width
	var_availableheight = this.calculateAvailableHeight();
	
	if (matchPercentage(String(this.mbr_availableHandleVertOffset)))
	{
		var_availablecentervertoffset = Math.floor((var_availableheight * parseFloat(this.mbr_availableHandleVertOffset.toString().chop()) )/100);
	}
	else
	{
		var_availablecentervertoffset = parseInt(this.mbr_availableHandleVertOffset);
	}
	
	return var_availablecentervertoffset;
};

RelativeLayer.prototype.calculateAvailableHorizOffset = function(par_visibleWidth, par_availableWidth)
{
	var var_visiblewidth;
	var var_availablewidth;
	var var_availablecenterhorizoffset;

	var var_availablehorizoffset;
	
	// get the visible width
	if (arguments.length < 1)
	{
		var_visiblewidth = this.calculateVisibleWidth();
	}
	else
	{
		var_visiblewidth = par_visibleWidth;
	}
	
	// get the available area width
	if (arguments.length < 2)
	{
		var_availablewidth = this.calculateAvailableWidth(var_visiblewidth);
	}
	else
	{
		var_availablewidth = par_availableWidth;
	}

	// get the available area center horizontal offset
	var_availablecenterhorizoffset = this.calculateAvailableHandleHorizOffset();
	
	// calcute the absolute available area vertical offset
	// check if the available area horizontal offset is just digits without any operators nor percentages
	if (matchNumber(String(this.mbr_availableHorizOffset)))
	{
		// only digits
		var_availablehorizoffset = parseInt(this.mbr_availableHorizOffset);
	}
	else
	{
		// with operators
		// concanate the percentage and pixel string to exactly 2 parts, first the percentage, then the pixels
		var var_concanatedstring = concanatePercentagePixels(this.mbr_availableHorizOffset);
		// separate the 2 parts and isolate the operator
		var var_matches = matchPercentagesAndNumbers(var_concanatedstring);
		// calculate the percentage according to the parent and add the fixed pixels to it
		var_availablehorizoffset = Math.floor((var_visiblewidth * parseFloat(var_matches[0]))/100) + parseInt(var_matches[1]);
	}
	
	// calcute the absolute available area horizontal offset relative to the visible border
	if (!(this.mbr_availableVertBorder == TOP))
	{
		var_availablehorizoffset = var_visiblewidth - (var_availablehorizoffset);
	}
	
	var_availablehorizoffset = var_availablehorizoffset - ( Math.floor(var_availablewidth/2) + var_availablecenterhorizoffset );

	if (var_availablehorizoffset > 0)
	// available area starts too far to the right
	{
		var_availablehorizoffset = 0;
	}
	else if ((var_availablehorizoffset + var_availablewidth) < var_visiblewidth)
	// available area starts too far to the left
	{
		var_availablehorizoffset = var_visiblewidth - var_availablewidth;
	}

	return var_availablehorizoffset;
};

RelativeLayer.prototype.calculateAvailableVertOffset = function(par_visibleHeight, par_availableHeight)
{
	var var_visibleheight;
	var var_availableheight;
	var var_availablevertoffset;
	var var_availablecentervertoffset;
	
	// get the visible height
	if (arguments.length < 1)
	{
		var_visibleheight = this.calculateVisibleHeight();
	}
	else
	{
		var_visibleheight = par_visibleHeight;
	}
	
	// get the available area height
	if (arguments.length < 2)
	{
		var_availableheight = this.calculateAvailableHeight(var_visibleheight);
	}
	else
	{
		var_availableheight = par_availableHeight;
	}

	// get the available area center vertical offset
	var_availablecentervertoffset = this.calculateAvailableHandleVertOffset();
	
	// calcute the absolute available area vertical offset
	// check if the available area vertical offset is just digits without any operators nor percentages
	if (matchNumber(String(this.mbr_availableVertOffset)))
	{
		// only digits
		var_availablevertoffset = parseInt(this.mbr_availableVertOffset);
	}
	else
	{
		// with operators
		// concanate the percentage and pixel string to exactly 2 parts, first the percentage, then the pixels
		var var_concanatedstring = concanatePercentagePixels(this.mbr_availableVertOffset);
		// separate the 2 parts and isolate the operator
		var var_matches = matchPercentagesAndNumbers(var_concanatedstring);
		// calculate the percentage according to the parent and add the fixed pixels to it
		var_availablevertoffset = Math.floor((var_visibleheight * parseFloat(var_matches[0]))/100) + parseInt(var_matches[1]);
	}
	
	// calcute the absolute available area vertical offset relative to the visible border
	if (!(this.mbr_availableVertBorder == TOP))
	{
		var_availablevertoffset = var_visibleheight - (var_availablevertoffset);
	}
	
	var_availablevertoffset = var_availablevertoffset - (Math.floor(var_availableheight/2) + var_availablecentervertoffset);

	if (var_availablevertoffset > 0)
	// available area starts too far to the bottom
	{
		var_availablevertoffset = 0;
	}
	else if ((var_availablevertoffset + var_availableheight) < var_visibleheight)
	// available area starts too far to the top
	{
		var_availablevertoffset = var_visibleheight - var_availableheight;
	}

	return var_availablevertoffset;
};

RelativeLayer.prototype.refresh = function()
{
	// refresh the background colors of the layers, this is not necessary for internet explorer
	// but it is for netscape since netscape loses some layer properties after a resize
	if (this.css)
	{
		if (this.mbr_zIndexChanged &&
		   this.mbr_zIndex != false)
		{
			this.css.zIndex = this.mbr_zIndex;
		}

		if (op)
		{
			if (this.mbr_bgColor &&
			   this.mbr_bgColorChanged)
			{
				this.css.background = this.mbr_bgColor;
				this.mbr_bgColorChanged = false;
			}
			if (this.mbr_bgImage &&
			   this.mbr_bgImageChanged)
			{
				// opera is unable to dynamically change a layer's background image
			}
			if (this.mbr_visible)
			{
				this.css.visibility = "visible";
			}
			else
			{
				this.css.visibility = "hidden";
			}
		}
		else if (mz || ie || kq)
		{
			if (this.mbr_bgColor &&
			   this.mbr_bgColorChanged)
			{
				this.css.backgroundColor = this.mbr_bgColor;
				this.mbr_bgColorChanged = false;
			}
			if (this.mbr_bgImage &&
			   this.mbr_bgImageChanged)
			{
				this.css.backgroundImage = "url("+this.mbr_bgImage+")";
				this.mbr_bgImageChanged = false;
			}
			if (this.mbr_visible)
			{
				this.css.visibility = "visible";
			}
			else
			{
				this.css.visibility = "hidden";
			}
		}
		else if (ns)
		{
			if (this.mbr_bgColor &&
			   this.mbr_bgColorChanged)
			{
				this.css.bgColor = this.mbr_bgColor;
				this.mbr_bgColorChanged = false;
			}
			if (this.mbr_bgImage &&
			   this.mbr_bgImageChanged)
			{
				this.css.background.src = this.mbr_bgImage;
				this.mbr_bgImageChanged = false;
			}
			if (this.mbr_visible)
			{
				this.css.visibility = "show";
			}
			else
			{
				this.css.visibility = "hide";
			}
		}
	}
};

RelativeLayer.prototype.visibleMoveTo = function(par_visibleHorizOffset, par_visibleVertOffset, par_fireEvent)
{
	if (arguments.length < 3)
	{
		par_fireEvent = true;
	}

	var var_oldhorizoffset = this.mbr_visibleHorizOffset;
	var var_oldvertoffset = this.mbr_visibleVertOffset;

	this.defineVisibleHorizOffset(par_visibleHorizOffset);
	this.defineVisibleVertOffset(par_visibleVertOffset);

	this.setPositioned(false);
	if (par_fireEvent)
	{
		this.positionVisibleMove("visiblemoveto", "oldhorizoffset", var_oldhorizoffset, "oldvertoffset", var_oldvertoffset);
	}
	else
	{
		this.positionVisibleMove(par_fireEvent);
	}
};

RelativeLayer.prototype.visibleMoveBy = function(par_visibleHorizOffset, par_visibleVertOffset, par_fireEvent)
{
	if (arguments.length < 3)
	{
		par_fireEvent = true;
	}

	var var_oldhorizoffset = this.mbr_visibleHorizOffset;
	var var_oldvertoffset = this.mbr_visibleVertOffset;

	this.visibleMoveBySetOffset(par_visibleHorizOffset, par_visibleVertOffset);
	
	this.setPositioned(false);
	if (par_fireEvent)
	{
		this.positionVisibleMove("visiblemoveby", "oldhorizoffset", var_oldhorizoffset, "oldvertoffset", var_oldvertoffset, "horizoffsetdiff", par_visibleHorizOffset, "vertoffsetdiff", par_visibleVertOffset);
	}
	else
	{
		this.positionVisibleMove(par_fireEvent);
	}
};

RelativeLayer.prototype.visibleMoveBySetOffset = function(par_visibleHorizOffset, par_visibleVertOffset)
{
	// check if the first character of the horizontal offset is a digit
	if (matchFirstCharDigit(String(par_visibleHorizOffset)))
	{
		// yes, prepend +
		par_visibleHorizOffset = "+"+par_visibleHorizOffset;
	}
	// check if the first character of the vertical offset is a digit
	if (matchFirstCharDigit(String(par_visibleVertOffset)))
	{
		// yes, prepend +
		par_visibleVertOffset = "+"+par_visibleVertOffset;
	}
	
	// calculate horizontal offset
	this.mbr_visibleHorizOffset += String(par_visibleHorizOffset);
	this.mbr_visibleHorizOffset = concanatePercentagePixels(this.mbr_visibleHorizOffset);

	// calculate vertical offset
	this.mbr_visibleVertOffset += String(par_visibleVertOffset);
	this.mbr_visibleVertOffset = concanatePercentagePixels(this.mbr_visibleVertOffset);
};

RelativeLayer.prototype.startScroll = function(par_x, par_y, par_speed)
{
	if (arguments.length < 3)
	{
		par_speed = 20;
	}

	this.mbr_scrollPreviousX = null;
	this.mbr_scrollPreviousY = null;
	this.mbr_scrollInterval = setInterval("getRelativeLayer(\""+this.getId()+"\").scroller("+par_x+", "+par_y+")", par_speed);

	return false;
};

RelativeLayer.prototype.stopScroll = function()
{
	if (this.mbr_scrollInterval)
	{
		clearInterval(this.mbr_scrollInterval);

		this.mbr_scrollPreviousX = null;
		this.mbr_scrollPreviousY = null;
		this.mbr_scrollInterval = null;
	}

	return false;
};

RelativeLayer.prototype.scroller = function(par_x, par_y)
{
	if (this.mbr_scrollPreviousX != this.mbr_x ||
	   this.mbr_scrollPreviousY != this.mbr_y)
	{
		this.mbr_scrollPreviousX = this.mbr_x;
		this.mbr_scrollPreviousY = this.mbr_y;
		this.availableMoveBy(par_x, par_y);
	}
	else
	{
		this.stopScroll();
	}
};

RelativeLayer.prototype.availableMoveTo = function(par_availableHorizOffset, par_availableVertOffset, par_fireEvent)
{
	if (arguments.length < 3)
	{
		par_fireEvent = true;
	}

	var var_oldhorizoffset = this.mbr_availableHorizOffset;
	var var_oldvertoffset = this.mbr_availableVertOffset;

	this.defineAvailableHorizOffset(par_availableHorizOffset);
	this.defineAvailableVertOffset(par_availableVertOffset);

	this.setPositioned(false);
	if (par_fireEvent)
	{
		this.position("availablemove", "oldhorizoffset", var_oldhorizoffset, "oldvertoffset", var_oldvertoffset, par_fireEvent);
	}
	else
	{
		this.position(par_fireEvent);
	}
};

RelativeLayer.prototype.availableMoveBy = function(par_availableHorizOffset, par_availableVertOffset)
{
	if (arguments.length < 3)
	{
		par_fireEvent = true;
	}

	var var_oldhorizoffset = this.mbr_availableHorizOffset;
	var var_oldvertoffset = this.mbr_availableVertOffset;

	this.availableMoveBySetOffset(par_availableHorizOffset, par_availableVertOffset);
	
	this.setPositioned(false);
	if (par_fireEvent)
	{
		this.position("availablemoveby", "oldhorizoffset", var_oldhorizoffset, "oldvertoffset", var_oldvertoffset, "horizoffsetdiff", par_availableHorizOffset, "vertoffsetdiff", par_availableVertOffset);
	}
	else
	{
		this.position(par_fireEvent);
	}
};

RelativeLayer.prototype.availableMoveBySetOffset = function(par_availableHorizOffset, par_availableVertOffset)
{
	var var_availablehorizoffset_concanated;
	var var_availablevertoffset_concanated;
	var var_availablehorizoffset_percentage;
	var var_availablehorizoffset_percentage_absolute;
	var var_availablehorizoffset_pixel;
	var var_availablevertoffset_percentage;
	var var_availablevertoffset_percentage_absolute;
	var var_availablevertoffset_pixel;
	var var_availablehorizoffset;
	var var_availablevertoffset;
	var var_availableheight;
	var var_availablewidth;
	var var_visibleheight;
	var var_visiblewidth;
	
	// concanate the offset expression to one percentage part and one pixel part
	var var_availablehorizoffset_concanated = concanatePercentagePixels(par_availableHorizOffset);
	var var_availablevertoffset_concanated = concanatePercentagePixels(par_availableVertOffset);
	
	// separate the 2 parts	and apply the sign modifiers
	var var_matches = matchPercentagesAndNumbers(var_availablehorizoffset_concanated);
	var_availablehorizoffset_percentage = parseFloat(var_matches[0]);
	var_availablehorizoffset_pixel = parseInt(var_matches[1]);
	var var_matches = matchPercentagesAndNumbers(var_availablevertoffset_concanated);
	var_availablevertoffset_percentage = parseFloat(var_matches[0]);
	var_availablevertoffset_pixel = parseInt(var_matches[1]);
	
	// calculate the absolute dimensions
	var_visibleheight = this.calculateVisibleHeight();
	var_visiblewidth = this.calculateVisibleWidth();
	var_availableheight = this.calculateAvailableHeight(var_visibleheight);
	var_availablewidth = this.calculateAvailableWidth(var_visiblewidth);
	var_availablevertoffset = this.calculateAvailableVertOffset(var_visibleheight, var_availableheight);
	var_availablehorizoffset = this.calculateAvailableHorizOffset(var_visiblewidth, var_availablewidth);
	
	// convert percentages to absolute pixel offsets
	var_availablehorizoffset_percentage_absolute = Math.floor((var_availablehorizoffset_percentage * var_visiblewidth) / 100);
	var_availablevertoffset_percentage_absolute = Math.floor((var_availablevertoffset_percentage * var_visibleheight) / 100);
	
	// limit the movement offsets to the limits of the visible area
	// horizontal limits
	if (var_availablehorizoffset_percentage != 0 || var_availablehorizoffset_pixel != 0)
	{
		// only pixel offset
		if (var_availablehorizoffset_percentage == 0)
		{
			// available area goes too far to the left
			if (var_availablehorizoffset + var_availablehorizoffset_pixel > 0)
			{
				var_availablehorizoffset_pixel = -1 * var_availablehorizoffset;
			}
			// available area goes too far to the right
			else if (var_availablewidth + var_availablehorizoffset + var_availablehorizoffset_pixel < var_visiblewidth )
			{
				var_availablehorizoffset_pixel = var_visiblewidth - var_availablewidth - var_availablehorizoffset;
			}
		}
		// only percentage offset
		else if (var_availablehorizoffset_pixel == 0)
		{
			// available area goes too far to the left
			if (var_availablehorizoffset + var_availablehorizoffset_percentage_absolute > 0)
			{
				// calculate the minimum horizontal percentage offset with a precision of 4 digits behind the comma
				var_availablehorizoffset_percentage = Math.floor(((-1 * var_availablehorizoffset) / (var_visiblewidth / 100)) * 10000) / 10000;
			}
			// available area goes too far to the right
			else if (var_availablewidth + var_availablehorizoffset + var_availablehorizoffset_percentage_absolute < var_visiblewidth )
			{
				// calculate the maximum horizontal percentage offset with a precision of 4 digits behind the comma
				var_availablehorizoffset_percentage = Math.floor(((var_visiblewidth - var_availablewidth - var_availablehorizoffset) / (var_visiblewidth / 100)) * 10000) / 10000;
			}
		}
		// both pixel offset and percentage offset
		else
		{
			// available area goes too far to the left
			if (var_availablehorizoffset + var_availablehorizoffset_percentage_absolute + var_availablehorizoffset_pixel > 0)
			{
				var_availablehorizoffset_pixel = -1 * var_availablehorizoffset - var_availablehorizoffset_percentage_absolute;
			}
			// available area goes too far to the right
			else if (var_availablewidth + var_availablehorizoffset + var_availablehorizoffset_percentage_absolute + var_availablehorizoffset_pixel < var_visiblewidth )
			{
				var_availablehorizoffset_pixel = var_visiblewidth - var_availablewidth - var_availablehorizoffset - var_availablehorizoffset_percentage_absolute;
			}
		}
	}
	// vertical limits
	if (var_availablevertoffset_percentage != 0 || var_availablevertoffset_pixel != 0)
	{
		// only pixel offset
		if (var_availablevertoffset_percentage == 0)
		{
			// available area goes too far to the bottom
			if (var_availablevertoffset + var_availablevertoffset_pixel > 0)
			{
				var_availablevertoffset_pixel = -1 * var_availablevertoffset;
			}
			// available area goes too far to the top
			else if (var_availableheight + var_availablevertoffset + var_availablevertoffset_pixel < var_visibleheight )
			{
				var_availablevertoffset_pixel = var_visibleheight - var_availableheight - var_availablevertoffset;
			}
		}
		// only percentage offset
		else if (var_availablevertoffset_pixel == 0)
		{
			// available area goes too far to the bottom
		  if (var_availablevertoffset + var_availablevertoffset_percentage_absolute > 0)
			{
				// calculate the minimum vertical percentage offset with a precision of 4 digits behind the comma
				var_availablevertoffset_percentage = Math.floor(((-1 * var_availablevertoffset) / (var_visibleheight / 100)) * 10000) / 10000;
			}
			// available area goes too far to the top
			else if (var_availableheight + var_availablevertoffset + var_availablevertoffset_percentage_absolute < var_visibleheight )
			{
				// calculate the maximum vertical percentage offset with a precision of 4 digits behind the comma
				var_availablevertoffset_percentage = Math.floor(((var_visibleheight - var_availableheight - var_availablevertoffset) / (var_visibleheight / 100)) * 10000) / 10000;
			}
		}
		// both pixel offset and percentage offset
		else
		{
			// available area goes too far to the bottom
			if (var_availablevertoffset + var_availablevertoffset_percentage_absolute + var_availablevertoffset_pixel > 0)
			{
				var_availablevertoffset_pixel = -1 * var_availablevertoffset - var_availablevertoffset_percentage_absolute;
			}
			// available area goes too far to the top
			else if (var_availableheight + var_availablevertoffset + var_availablevertoffset_percentage_absolute + var_availablevertoffset_pixel < var_visibleheight )
			{
				var_availablevertoffset_pixel = var_visibleheight - var_availableheight - var_availablevertoffset - var_availablevertoffset_percentage_absolute;
			}
		}
	}
	
	// define the new horizontal offset
	this.defineAvailableHorizOffset(
		concanatePercentagePixels(
			this.mbr_availableHorizOffset+
			// prepend a '+' sign if the percentage is not negative
			(var_availablehorizoffset_percentage>=0 ? '+' : '')+
			var_availablehorizoffset_percentage+'%'+
			// prepend a '+' sign if the pixels are not negative
			(var_availablehorizoffset_pixel>=0 ? '+' : '')+
			var_availablehorizoffset_pixel
			)
		);
	
	// define the new vertical offset
	this.defineAvailableVertOffset(
		concanatePercentagePixels(
			this.mbr_availableVertOffset+
			// prepend a '+' sign if the percentage is not negative
			(var_availablevertoffset_percentage>=0 ? '+' : '')+
			var_availablevertoffset_percentage+'%'+
			// prepend a '+' sign if the pixels are not negative
			(var_availablevertoffset_pixel>=0 ? '+' : '')+
			var_availablevertoffset_pixel
			)
		);
};

RelativeLayer.prototype.visibleResizeBy = function(par_widthOffset, par_heightOffset, par_fireEvent)
{
	if (arguments.length < 3)
	{
		par_fireEvent = true;
	}

	var var_oldwidth = this.mbr_visibleWidth;
	var var_oldheight = this.mbr_visibleHeight;

	this.defineVisibleWidth(
		concanatePercentagePixels(this.mbr_visibleWidth+
			((par_widthOffset.toString().charAt(0)!='-' && par_widthOffset.toString().charAt(0))!='-' ? '+' : '')+
			par_widthOffset)
	);
	this.defineVisibleHeight(
		concanatePercentagePixels(this.mbr_visibleHeight+
			((par_heightOffset.toString().charAt(0)!='-' && par_heightOffset.toString().charAt(0))!='-' ? '+' : '')+
			par_heightOffset)
	);

	this.setPositioned(false);
	if (par_fireEvent)
	{
		this.position("visibleresizeby", "oldwidth", var_oldwidth, "oldheight", var_oldheight, "widthdiff", par_widthOffset, "heightdiff", par_heightOffset);
	}
	else
	{
		this.position(par_fireEvent);
	}
};

RelativeLayer.prototype.visibleResizeTo = function(par_visibleWidth, par_visibleHeight, par_fireEvent)
{
	if (arguments.length < 3)
	{
		par_fireEvent = true;
	}

	var var_oldwidth = this.mbr_visibleWidth;
	var var_oldheight = this.mbr_visibleHeight;

	this.defineVisibleWidth(par_visibleWidth);
	this.defineVisibleHeight(par_visibleHeight);

	this.setPositioned(false);
	if (par_fireEvent)
	{
		this.position("visibleresizeto", "oldwidth", var_oldwidth, "oldheight", var_oldheight);
	}
	else
	{
		this.position(par_fireEvent);
	}
};

RelativeLayer.prototype.availableResizeBy = function(par_widthOffset, par_heightOffset, par_fireEvent)
{
	if (arguments.length < 3)
	{
		par_fireEvent = true;
	}

	var var_oldwidth = this.mbr_availableWidth;
	var var_oldheight = this.mbr_availableHeight;

	this.defineAvailableWidth(
		concanatePercentagePixels(this.mbr_availableWidth+
			((par_widthOffset.toString().charAt(0)!='-' && par_widthOffset.toString().charAt(0))!='-' ? '+' : '')+
			par_widthOffset)
	);
	this.defineAvailableHeight(
		concanatePercentagePixels(this.mbr_availableHeight+
			((par_heightOffset.toString().charAt(0)!='-' && par_heightOffset.toString().charAt(0))!='-' ? '+' : '')+
			par_heightOffset)
	);

	this.setPositioned(false);
	if (par_fireEvent)
	{
		this.position("availableresize", "oldwidth", var_oldwidth, "oldheight", var_oldheight, "widthdiff", par_widthOffset, "heightdiff", par_heightOffset);
	}
	else
	{
		this.position(par_fireEvent);
	}
};

RelativeLayer.prototype.availableResizeTo = function(par_availableWidth, par_availableHeight, par_fireEvent)
{
	if (arguments.length < 3)
	{
		par_fireEvent = true;
	}

	var var_oldwidth = this.mbr_availableWidth;
	var var_oldheight = this.mbr_availableHeight;

	this.defineAvailableWidth(par_availableWidth);
	this.defineAvailableHeight(par_availableHeight);

	this.setPositioned(false);
	if (par_fireEvent)
	{
		this.position("availableresize", "oldwidth", var_oldwidth, "oldheight", var_oldheight);
	}
	else
	{
		this.position(par_fireEvent);
	}
};

RelativeLayer.prototype.getAbsoluteX = function()
{
	return parseInt(this.mbr_x);
};

RelativeLayer.prototype.getAbsoluteY = function()
{
	return parseInt(this.mbr_y);
};

RelativeLayer.prototype.getAbsolutePageX = function()
{
	var var_x = 0;
	var var_node = this.mbr_node;
	
	if (op)
	{
		while (var_node &&
			  var_node.style)
		{
			var_x += var_node.style.pixelLeft;
			var_node = var_node.offsetParent;
		}
	}
	else if (mz || kq)
	{
		while (var_node &&
			  var_node.style &&
			  var_node.style.left.length > 0)
		{
			var_x += parseInt(var_node.style.left);
			var_node = var_node.parentNode;
		}
	}
	else if (ns)
	{
		var_x = this.css.pageX;
	}
	else if (ie)
	{
		while (var_node &&
			  var_node.style)
		{
			var_x += var_node.style.pixelLeft;
			var_node = var_node.parentElement;
		}
	}

	return var_x;
};

RelativeLayer.prototype.getAbsolutePageY = function()
{
	var var_y = 0;
	var var_node = this.mbr_node;

	if (op)
	{
		while (var_node != null &&
			  var_node.style)
		{
			var_y += var_node.style.pixelTop;
			var_node = var_node.offsetParent;
		}
	}
	else if (mz || kq)
	{
		while (var_node != null &&
			  var_node.style &&
			  var_node.style.top.length > 0)
		{
			var_y += parseInt(var_node.style.top);
			var_node = var_node.parentNode;
		}
	}
	else if (ns)
	{
		var_y = this.css.pageY;
	}
	else if (ie)
	{
		while (var_node &&
			  var_node.style)
		{
			var_y += var_node.style.pixelTop;
			var_node = var_node.parentElement;
		}
	}

	return var_y;
};

RelativeLayer.prototype.absoluteMoveBy = function(par_x, par_y)
{
	var var_oldx = this.mbr_x;
	var var_oldy = this.mbr_y;

	this.mbr_x = this.getAbsoluteX() + par_x;
	this.css.left = this.getAbsoluteX();
	this.mbr_y = this.getAbsoluteY() + par_y;
	this.css.top = this.getAbsoluteY();
	// notify event listeners
	this.fireEvent("absolutemoveby", "oldx", var_oldx, "oldy", var_oldy, "xdiff", par_x, "ydiff", par_y);
};

RelativeLayer.prototype.absoluteMoveTo = function(par_x, par_y)
{
	var var_oldx = this.mbr_x;
	var var_oldy = this.mbr_y;

	this.mbr_x = par_x;
	this.css.left = this.getAbsoluteX();
	this.mbr_y = par_y;
	this.css.top = this.getAbsoluteY();
	// notify event listeners
	this.fireEvent("absolutemoveto", "oldx", var_oldx, "oldy", var_oldy);
};

RelativeLayer.prototype.absoluteResizeTo = function(par_width, par_height)
{
	var var_oldwidth;
	var var_oldheight;
	
	if (par_width < 0)
	{
		par_width = 0;
	}
	if (par_height < 0)
	{
		par_height = 0;
	}
	if (op)
	{
		var_oldwidth = this.css.pixelWidth;
		var_oldheight = this.css.pixelHeight;

		this.css.pixelWidth = par_width;
		if (arguments.length > 1)
		{
			this.css.pixelHeight = par_height;
		}
	}
	else if (mz || kq)
	{
		var_oldwidth = this.mbr_node.offsetWidth;
		var_oldheight = this.mbr_node.offsetHeight;

		this.css.width = par_width;
		if (arguments.length > 1)
		{
			this.css.height = par_height;
		}
	}
	else if (ns)
	{
		if (this.mbr_autoAvailableWidth)
		{
			var_oldwidth = this.mbr_availableWidth;
		}
		else
		{
			var_oldwidth = this.mbr_previousAbsoluteWidth;
		}
		if (this.mbr_autoAvailableHeight)
		{
			var_oldheight = this.mbr_availableHeight;
		}
		else
		{
			var_oldheight = this.mbr_previousAbsoluteHeight;
		}

		// adapt the wrapping width of the dynamically set html to the new wrapping width
		if (this.mbr_sethtml &&
		   par_width != this.css.width)
		{
			this.setHtml(this.mbr_sethtml);
		}
		if (arguments.length > 1)
		{
			this.css.resizeTo(par_width, par_height);
		}
		this.mbr_previousAbsoluteWidth = par_width;
		this.mbr_previousAbsoluteHeight = par_height;
	}
	else if (ie)
	{
		var_oldwidth = this.mbr_node.offsetWidth;
		var_oldheight = this.mbr_node.offsetHeight;

		this.css.width = par_width;
		if (arguments.length > 1)
		{
			this.css.height = par_height;
		}
	}
	// notify event listeners
	this.fireEvent("absoluteresize", "oldwidth", var_oldwidth, "oldheight", var_oldheight, "newwidth", par_width, "newheight", par_height);
};

RelativeLayer.prototype.absoluteClipTo = function(par_top, par_right, par_bottom, par_left)
{
	var var_oldtop;
	var var_oldright;
	var var_oldbottom;
	var var_oldleft;

	if (this.mbr_previousclip)
	{
		var var_rect = splitRect(this.mbr_previousclip);

		var_oldtop = var_rect[1];
		var_oldright = var_rect[2];
		var_oldbottom = var_rect[3];
		var_oldleft = var_rect[4];
	}

	this.mbr_previousclip = "rect("+par_top+"px "+par_right+"px "+par_bottom+"px "+par_left+"px)";

	if (op || mz || ie || kq)
	{
		this.css.clip = "rect("+par_top+"px "+par_right+"px "+par_bottom+"px "+par_left+"px)";
	}
	else if (ns)
	{
		this.css.clip.top = par_top;
		this.css.clip.right = par_right;
		this.css.clip.bottom = par_bottom;
		this.css.clip.left = par_left;
	}

	// notify event listeners
	this.fireEvent("absoluteclip", "oldtop", var_oldtop, "oldright", var_oldright, "oldbottom", var_oldbottom, "oldleft", var_oldleft, "newtop",par_top , "newright", par_right, "newbottom", par_bottom, "newleft", par_left);
};

RelativeLayer.prototype.positionVisibleMove = function()
{
	if (arguments.length < 1)
	{
		arguments[0] = "position";
	}

	var var_visiblewidth;
	var var_visibleheight;
	var var_visiblehorizoffset;
	var var_visiblevertoffset;
	var var_availablehorizoffset;
	var var_availablevertoffset;
	
	if (!this.mbr_dragActive &&
	   !this.isPositioned())
	{
		// position dependecies before continuing the calculations
		this.positionDependencies();
		
		if (!this.isPositioned())
		{
			// update the positioning validity flag
			this.setPositioned(true);
			
			// calculate the visible dimensions
			var_visiblewidth = this.calculateVisibleWidth();
			var_visibleheight = this.calculateVisibleHeight();
			
			// get the absolute position of the visible area
			var_visiblehorizoffset = this.calculateVisibleHorizOffset(var_visiblewidth);
			var_visiblevertoffset = this.calculateVisibleVertOffset(var_visibleheight);
			
			// get the absolute position of the available area
			var_availablehorizoffset = this.calculateAvailableHorizOffset(var_visiblewidth, -1);
			var_availablevertoffset = this.calculateAvailableVertOffset(var_visibleheight, -1);
			
			// move the visible area to the calculated position
			this.absoluteMoveTo(var_visiblehorizoffset+var_availablehorizoffset,var_visiblevertoffset+var_availablevertoffset);
			
			// refresh the layer
			this.refresh();
			
			// update the document dimensions
			updateDocumentDimensions(this);
			
			// notify event listeners if an event can be launched
			if (typeof(arguments[0]) != "boolean" || arguments[0] == true)
			{
				var var_eventobject = this.makeEventObject(arguments);
				this.fireEventObject(var_eventobject);
			}
			
			// position the layers that depend on this one
			this.positionDependingLayers();

			// should the document size be changed due to for example scrollbars additions,
			// consider it as a windows resize
			if (checkResize())
			{
				processResizeHooks();
				storeWindowDimensions();
			}
		}
	}
};

RelativeLayer.prototype.position = function()
{
	if (arguments.length < 1)
	{
		arguments[0] = "position";
	}

	var var_visiblewidth;
	var var_visibleheight;
	var var_visiblehorizoffset;
	var var_visiblevertoffset;
	var var_availablewidth;
	var var_availableheight;
	var var_availablehorizoffset;
	var var_availablevertoffset;

	if (this.css &&
	   !this.mbr_dragActive)
	{
		// position dependecies before continuing the calculations
		this.positionDependencies();

		if (!this.isPositioned())
		{
			// update the positioning validity flag
			this.setPositioned(true);

			// get the area widths
			this.updateAutoVisibleWidth();
			var_visiblewidth = this.calculateVisibleWidth();
			this.updateAutoAvailableWidth();
			var_availablewidth = this.calculateAvailableWidth(var_visiblewidth);
			
			// get the area heights
			if (this.mbr_autoVisibleHeight)
			{
				if (!this.mbr_autoVisibleWidth)
				{
					this.absoluteResizeTo(var_availablewidth);
					if (ie && glb_version == 4)
					{
						// something should be done here to make internet explorer 4 update the layout before continuing
						// the only known working way is to delay the rest with a setTimeout() but this would mess up the
						// whole structure completely, so it's not being implemented like that
					}
				}
				this.updateAutoVisibleHeight();
			}
			var_visibleheight = this.calculateVisibleHeight();
			if (this.mbr_autoAvailableHeight)
			{
				if (!(this.mbr_autoVisibleWidth && var_visiblewidth == var_availablewidth) &&
				   !this.mbr_autoAvailableWidth)
				{
					this.absoluteResizeTo(var_availablewidth);
					if (ie && glb_version == 4)
					{
						// something should be done here to make internet explorer 4 update the layout before continuing
						// the only known working wayis  to delay the rest with a setTimeout() but this would mess up the
						// whole structure completely, so it's not being implemented like that
					}
				}
				this.updateAutoAvailableHeight();
			}
			var_availableheight = this.calculateAvailableHeight(var_visibleheight);
			
			// get the absolute position of the visible area
			var_visiblehorizoffset = this.calculateVisibleHorizOffset(var_visiblewidth);
			var_visiblevertoffset = this.calculateVisibleVertOffset(var_visibleheight);
			
			// get the absolute position of the available area
			var_availablehorizoffset = this.calculateAvailableHorizOffset(var_visiblewidth, var_availablewidth);
			var_availablevertoffset = this.calculateAvailableVertOffset(var_visibleheight, var_availableheight);
			
			// bug in internet explorer 4.x that doesn't position a child layer correctly if the parent hasn't changed size
			if (ie)
			{
				if ((this.css.pixelWidth == var_availablewidth) &&
				   (this.css.pixelHeight == var_availableheight))
				{
					this.absoluteResizeTo(var_availablewidth+1, var_availableheight);
				}
			}
			// resize the available	area only if the height is not updated automatically
			if (!(this.mbr_autoVisibleWidth && var_visiblewidth == var_availablewidth) &&
			   !this.mbr_autoAvailableWidth &&
			   !(this.mbr_autoVisibleHeight && var_visibleheight == var_availableheight) &&
			   !this.mbr_autoAvailableHeight)
			{
				this.absoluteResizeTo(var_availablewidth, var_availableheight);
			}
			
			// resize the visible area
			this.absoluteClipTo(-var_availablevertoffset, -var_availablehorizoffset+var_visiblewidth, -var_availablevertoffset+var_visibleheight, -var_availablehorizoffset);
			
			// move the visible area to the calculated position
			this.absoluteMoveTo(var_visiblehorizoffset+var_availablehorizoffset, var_visiblevertoffset+var_availablevertoffset);
			
			// refresh the layer
			this.refresh();
			
			// update the document dimensions
			updateDocumentDimensions(this);
			
			// notify event listeners if an event can be launched
			if (typeof(arguments[0]) != "boolean" || arguments[0] == true)
			{
				var var_eventobject = this.makeEventObject(arguments);
				this.fireEventObject(var_eventobject);
			}
			
			// position the layers that depend on this one
			this.positionDependingLayers();

			// should the document size be changed due to for example scrollbars additions,
			// consider it as a windows resize
			if (checkResize())
			{
				processResizeHooks();
				storeWindowDimensions();
			}
		}
	}
};
